//c 24/03/2022: use joystick
//d 26/03/2022: add click , wheel
//e 29/03/2022: on click piedzo led, or during wheel mode, piedzo on/,,,/off led, wheel / y 
//g 26/04/2022: use a box with four button
//h 12/05/2022: led on during long press, use ezButton
//k 09/06/2022: mouse speed modification 
//@fr test double click

#include "Mouse.h"   
#include "pitches.h"
#include <ezButton.h>
     
#define USE_JOYSTICK        1
#define USE_DEBUG_JOYSTICK  0 
#define USE_MOUSE           1
#define USE_BIP_ON_BUTTON   0

#define USE_BUTTON  1
//@ todo

// define button pin
const int pin_left  = 5;
const int pin_right = 4;
const int pin_wheel = 3;
const int pin_press = 2;

const int pin_led_left  =  8; //red
const int pin_led_right = 10; //green
const int pin_led_wheel = 11; //yellow
const int pin_led_press = 12; //blue

// define piedzo pin
const int pin_piedzo=9;

// define response delay of the mouse, in ms
const int responseDelay = 20;

// notes in the melody:
const int melody[] = {
  NOTE_C4, NOTE_G3, NOTE_G3, NOTE_A3, NOTE_G3, 0, NOTE_B3, NOTE_C4
};

// note durations: 4 = quarter note, 8 = eighth note, etc.:
const int noteDurations[] = {
  4, 8, 8, 4, 4, 4, 4, 4
};

//define analog pin for joystick
#define Axe_X A0
#define Axe_Y A1
    
// define state buttons
bool bt_left  = LOW;
bool bt_right = LOW;
bool bt_wheel = LOW;
bool bt_press = LOW;

// define previous state buttons
bool btp_left  = bt_left;
bool btp_right = bt_right;
bool btp_wheel = bt_wheel;
bool btp_press = bt_press;

// set wheel button state
bool state_wheel = false;
bool state_press = false;
bool state_press_left = false;
bool state_press_right = false;

// parameters for reading the joystick:
const int center        = 1024 / 2;           // resting position value
const int threshold_min = center-50;         // min resting threshold
const int threshold_max = center+50;         // max resting threshold

int xMin=1024,xMax=0,yMin=1024,yMax=0;

ezButton button_left(pin_left);
ezButton button_right(pin_right);
ezButton button_wheel(pin_wheel);
ezButton button_press(pin_press);

//*********************************************

void buzzer_right(void)  {
#if USE_BIP_ON_BUTTON
      buzzer(4,5);
#endif
}

void buzzer_left(void)  {
#if USE_BIP_ON_BUTTON
      buzzer(1,2);
#endif
}

void buzzer(int nStart,int nStop)
{
      // iterate over the notes of the melody:
      for (int thisNote = nStart; thisNote < nStop; thisNote++) {
 
        int noteDuration = 1000 / noteDurations[thisNote];
        tone(pin_piedzo, melody[thisNote], noteDuration);

        int pauseBetweenNotes = noteDuration * 1.3;
        delay(pauseBetweenNotes);

        noTone(pin_piedzo);
      }
}

//*********************************************

int toogle_press(void) {
  int result=0;

  //when the button press is pressed, toogle between press/release state
  if (state_press==false) {
    digitalWrite(pin_led_press, HIGH);  // indicate via LED
    Serial.println("(button press)--------------> PRESS");
    #if USE_BIP_ON_BUTTON
        buzzer(4,5);
    #endif
    state_press=true;
  }
  else {
    digitalWrite(pin_led_press, LOW);  // stop LED
    Serial.println("(button press)--------------> RELEASE");
    #if USE_BIP_ON_BUTTON
      buzzer(7,8);
    #endif
    state_press=false;
  }
  result=state_press;
  return result;
}

int toogle_wheel(void){
  int result=0;

  //when the wheel button is pressed, toogle between xy/wheel state
  if (state_wheel==false) {
    digitalWrite(pin_led_wheel, HIGH);  // indicate via LED
    Serial.println("(xy/wheel)--------------> MODE WHEEL");
    #if USE_BIP_ON_BUTTON
      buzzer(0,5);
    #endif
    state_wheel=true;
  }
  else {
    digitalWrite(pin_led_wheel, LOW);  // stop LED
    Serial.println("(xy/wheel)--------------> MODE XY");
    #if USE_BIP_ON_BUTTON      
      buzzer(6,8);
    #endif
    state_wheel=false;
  }
  result=state_wheel;
  return result;
}

//*********************************************

bool  action_button_left(){
  int result=0;

  if (state_press==false) {
    Serial.println("(button left)--------------> click ");
  
    digitalWrite(pin_led_left, HIGH);
    buzzer_left();
    digitalWrite(pin_led_left, LOW);
    
    #if USE_MOUSE
      Mouse.click(MOUSE_LEFT);
    #endif
  }
  else
  {
    Serial.println("(button left)--------------> in press mode");
    if ((state_press_left==false) && (state_press_right==false)) {
      Serial.println("(button left)--------------> send press");
      digitalWrite(pin_led_left, HIGH);
      #if USE_MOUSE
        Mouse.press(MOUSE_LEFT);
      #endif
      state_press_left=true;
      state_press_right=false;
    }
    else if (state_press_left==true) {
      Serial.println("(button left)--------------> send release");
      digitalWrite(pin_led_left, LOW);
      #if USE_MOUSE
        Mouse.release(MOUSE_LEFT);
      #endif
      digitalWrite(pin_led_press, LOW);  // stop LED
      state_press_left=false;
      state_press_right=false;
      toogle_press();
    }
  }
  
  return result;
}


bool action_button_right(){
  int result=0;

  if (state_press==false) {
    Serial.println("(button right)-------------> click ");
  
    digitalWrite(pin_led_right, HIGH);
    buzzer_right();
    digitalWrite(pin_led_right, LOW);
    
    #if USE_MOUSE
      Mouse.click(MOUSE_RIGHT);
    #endif
  }
  else
  {
    Serial.println("(button right)---------------> in press mode");
    
    if ((state_press_left==false) && (state_press_right==false)) {
      Serial.println("(button right)--------------> send press");
      digitalWrite(pin_led_right, HIGH);
      digitalWrite(pin_led_press, HIGH);
      #if USE_MOUSE
        Mouse.press(MOUSE_RIGHT);
      #endif
      state_press_left=false;
      state_press_right=true;
    }
    else if (state_press_right==true) {
      Serial.println("(button right)--------------> send release");
      #if USE_MOUSE
        Mouse.release(MOUSE_RIGHT);
      #endif
      digitalWrite(pin_led_right, LOW);
      digitalWrite(pin_led_press, LOW);
      state_press_left=false;
      state_press_right=false;
      toogle_press();
    }
  }
  
  return result;
}

//*********************************************

int readAxis(int thisAxis) {

  // read the analog input:
  unsigned int reading = analogRead(thisAxis);

  reading = 1023 - reading;
  
  // return the reading for this axis value:
  return reading;
}

signed char getMove(int axisPosition) {
  
  signed char ret = 0;

       if ((axisPosition > threshold_min)&&(axisPosition < threshold_max))  { ret = 0; }
  else if ((axisPosition>threshold_max)&&(axisPosition<1010))   { ret = map(axisPosition,threshold_max,1010,1,4); }
  else if (axisPosition>1010)     { ret = map(axisPosition,1010,1023,10,12); }
  else if ((axisPosition<threshold_min)&&(axisPosition>10))    { ret = map(axisPosition,10,threshold_min,-4,-1); }
  else if (axisPosition<10)     { ret = map(axisPosition,0,10,-12,-10); }
  return ret;
}

z//*********************************************

void setup() {
  Serial.begin(9600);
  //while(!Serial)     ;
    
  //pinMode(pin_left,  INPUT_PULLUP);

  button_left.setDebounceTime(50);
  button_right.setDebounceTime(50);
  button_wheel.setDebounceTime(50);
  button_press.setDebounceTime(50);

 
  // initialize the LED pin as an output:
  pinMode(pin_led_left,  OUTPUT);
  pinMode(pin_led_right, OUTPUT);
  pinMode(pin_led_wheel, OUT
  
  // use A0 A1 analog pin for joystick axis
  
  // setup default state
  state_wheel = false;
  state_press = false;
    
  //take control of the mouse:
  #if USE_MOUSE
    Serial.println("Mouse functions enable for 32u4 or SAMD");
    Mouse.begin();
  #endif

}

//*********************************************

void loop() {
  
  // *** polling mode : ***

  button_left.loop();
  button_right.loop();
  button_wheel.loop();
  button_press.loop();
  Serial.println(A0);
  Serial.println(A1);

  //  Serial.println(button_wheel.getState());
  //when the wheel button is pressed, toogle between xy/wheel state 
  if(button_wheel.isPressed()) {
    Serial.println("(button wheel)-------------->");
    toogle_wheel();
  }

  if (state_wheel==false) {
    //Serial.println(state_wheel);
    if(button_press.isPressed())
      toogle_press();
      //when the press button is pressed, toogle between click/long press state
        
    if(button_left.isPressed())
      action_button_left();
  
    if(button_right.isPressed())
      action_button_right(); 
  }
  else {
  
  }
  
  //read & display axe x&y value
  #if USE_JOYSTICK

    // read the two axes:
    int xReading = readAxis(Axe_X);
    int yReading = readAxis(Axe_Y);
  
    // scale the two axes:
    int xScale = getMove(xReading);
    int yScale = getMove(yReading);
    
    if (xScale==1)  {delay(200);}
    
    // update x y, min max:
    if (xReading<xMin) {xMin=xReading;}
    if (xReading>xMax) {xMax=xReading;}
    
    if (yReading<yMin) {yMin=yReading;}
    if (yReading>yMax) {yMax=yReading;}
    
    #if USE_DEBUG_JOYSTICK
    Serial.print(xMin);
    Serial.print(" ");
    Serial.print(xReading);
    Serial.print(" ");   
    Serial.print(xMax);
    Serial.print(" ");
    
    Serial.print(yMin);
    Serial.print(" ");
    Serial.print(yReading);
    Serial.print(" ");
    Serial.print(yMax);
    Serial.println(" ");
    
    #endif 
    
    #if USE_MOUSE
      if (state_wheel) {
        
        //use y as wheel
        if (yScale>0) { Mouse.move(0,0, 1);} else
        if (yScale<0) { Mouse.move(0,0, -1);}

        delay(20);
      }
      else {
        if ((xScale != 0)||(yScale!=0)) {
          Mouse.move(xScale, -yScale, 0);
        }
      }
    #endif
  
  #endif
    
  delay(responseDelay);
}
